// -*- mode: C++; c-file-style: "Stroustrup" -*-
//
// This file is part of krot,
// a program for the simulation, assignment and fit of HRLIF spectra.
//
// Copyright (C) 1998 Jochen Kpper
//
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
// more details.
//
// You should have received a copy of the GNU General Public License along with
// this program; see the file License. if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 , USA
//
// If you use this program for your scientific work, please cite it according to
// the file CITATION that comes with the package.
//
// $Id: workarea.cc,v 1.8 1998/10/09 14:49:20 jochen Exp $



#include "drawarea.h"
#include "krot.h"
#include "krotData.h"
#include "scrollbar64.h"
#include "workarea.h"

#include <qlayout.h>

#include <sys/types.h>


WorkArea::WorkArea( KRotData *krotdata, QWidget *parent, const char *name )
    : QWidget( parent, name ),
      average( krotdata->averaging() ),
      first ( 0 ),
      krotData( krotdata ),
      last( 0 )
{
    KROT_LAUNCH( "Launching WorkArea constructor." );
    QBoxLayout *layout = new QVBoxLayout( this );
    drawarea = new DrawArea( krotData, this, "DrawArea" );
    layout->addWidget( drawarea );
    scrollbar = new ScrollBar64( 0, 0, 1, width() / 2, 0, ScrollBar64::Horizontal, this, "ScrollBar" );
    scrollbar->setFixedHeight( 15 );
    layout->addWidget( scrollbar );
    layout->activate();

    connect( drawarea, SIGNAL( freq( const int64_t ) ),
	     this,     SLOT( setFreq( const int64_t ) ) );
    connect( drawarea, SIGNAL( assignment( const AssignedFrequency& ) ),
	     this,     SLOT( newAssignment( const AssignedFrequency& ) ) );
    connect( scrollbar, SIGNAL( valueChanged( int64_t ) ),
	     drawarea,  SLOT( setStart( int64_t ) ) );

    return;
}



void WorkArea::limits( int64_t start, int64_t stop )
{
    KROT_LAUNCH( "Launching WorkArea::limits" );
    int width = drawarea->width();
    first = start;
    last = stop;
    int64_t beg = start / average + width / 2;
    int64_t end = stop / average - width / 2;
    scrollbar->setRange( beg, end >= beg ? end : beg );
    scrollbar->setSteps( static_cast< int64_t >( 1 ),
			 static_cast< int64_t >( width / 2 ) );
    update();
    return;
}



void WorkArea::setAveraging( const unsigned int newAveraging )
{
    KROT_LAUNCH( "Launching WorkArea::setAveraging" );
    int64_t val = static_cast< int64_t >( static_cast< double >(average) / newAveraging
					  * scrollbar->value() );
    average = newAveraging;
    limits( first, last );
    scrollbar->setValue( val );
    return;
}



void WorkArea::setPosition( const int64_t pos )
{
    KROT_LAUNCH( "Launching WorkArea::setPosition" );
    scrollbar->setValue( pos );
    return;
}
